package cn.estsh.i3plus.ext.mes.apiservice.dbinterface;

import cn.estsh.i3plus.mes.apiservice.serviceimpl.engine.script.EngineScriptManager;
import cn.estsh.i3plus.mes.apiservice.util.datatable.DataTable;
import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.util.List;
import java.util.Map;

/**
 * @Description : 接口数据读取
 * 按照配置规则调用 DAO 层读取数据
 * @Reference :
 * @Author : rock.yu
 * @CreateDate : 2019-06-11 19:34
 * @Modify: 1. 2019-10-14 19:13 增加了调用脚本引擎处理数据的逻辑
 * 1.
 **/
@Component
@NoArgsConstructor
public class WmsSAPDbReader extends WmsSAPDbSyncBase {
    public static final Logger LOGGER = LoggerFactory.getLogger(WmsSAPDbReader.class);

    // 动态数据源，靠外部调用类传入此对象
    private DynamicDataSourceProxy dataSourceProxy;

    // 数据库读写用的数据源
    @Getter
    @Setter
    private Connection srcConn;

    /**
     * 使用脚本过滤之前的原始数据集要保留下来，有肯能用这个数据集回写同步标识
     */
    @Getter
    private List<Map<String, Object>> originReadData = null;

    // 脚本引擎
    @Autowired
    private EngineScriptManager engineScriptManager;

    public WmsSAPDbReader(DynamicDataSourceProxy dataSourceProxy) {
        this.dataSourceProxy = dataSourceProxy;
    }

    public WmsSAPDbReader(MesInterfaceDataMapper dataMapper) {
        super(dataMapper);
    }

    /**
     * 读取数据
     * @return 读取到的接口表数据
     * @throws Exception 接口映射关系不存在时抛出异常
     */
    public List<Map<String, Object>> readData() throws Exception {
        List<Map<String, Object>> resultList;

        this.originReadData = null;
        if (this.dataMapper == null) {
            throw new IllegalArgumentException("not set data mapper!");
        }

        // 配置了使用脚本引擎获取数据
        if (this.dataMapper.getUseScriptPull() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
            resultList = getDataByScript();
        } else {
            resultList = getDataBySQL();
        }

        // 配置了使用脚本引擎过滤来源数据
        if (this.dataMapper.getUseScriptFilter() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
            // 保留原始数据集
            this.originReadData = resultList;
            resultList = getDataByFilter(resultList);
        }

        return resultList;
    }

    /**
     * 执行 SQL 语句获取原始数据
     * @return 原始数据
     * @throws Exception 数据库异常
     */
    private List<Map<String, Object>> getDataBySQL() throws Exception {
        // 构建查询语句
        String sqlString = buildQuerySQL(isMSSQL());
        LOGGER.info("WmsSAPDbReader.getDataBySQL:{}", sqlString);
        return dataSourceProxy.queryMapList(sqlString, srcConn);
    }

    /**
     * 根据映射表的配置参数构建查询数据的 SQL 语句
     * @param isMSSql 是否 MSSQL 数据库，目前只支持了 MSSQL 和 MYSQL 的判断
     * @return 查询 SQL
     */
    public String buildQuerySQL(boolean isMSSql) {
        String sqlString;

        if (isMSSql) {
            if (StringUtils.isEmpty(this.dataMapper.getSrcGroupColumns())) {
                sqlString = "select top " + this.dataMapper.getSrcGetLimit()
                        + " * from " + this.dataMapper.getSrcTableName() + " model "
                        + " where " + this.dataMapper.getSrcWhere()
                        + " order by " + this.dataMapper.getSrcOrderBy();
            } else {
                sqlString = "select * from " + this.dataMapper.getSrcTableName() + " model "
                        + " where " + this.dataMapper.getSrcWhere()
                        + " and " + this.dataMapper.getSrcGroupColumns() + " in ( "
                        + " select top " + this.dataMapper.getSrcGetLimit() + " "
                        + this.dataMapper.getSrcGroupColumns()
                        + " from " + this.dataMapper.getSrcTableName()
                        + " where " + this.dataMapper.getSrcWhere()
                        + " group by " + this.dataMapper.getSrcGroupColumns()
                        + " )" + " order by " + this.dataMapper.getSrcOrderBy();
            }
        } else {
            if (StringUtils.isEmpty(this.dataMapper.getSrcGroupColumns())) {
                sqlString = "select * from " + this.dataMapper.getSrcTableName() + " model "
                        + " where " + this.dataMapper.getSrcWhere()
                        + " order by " + this.dataMapper.getSrcOrderBy()
                        + " limit " + this.dataMapper.getSrcGetLimit();
            } else {
                sqlString = "select * from " + this.dataMapper.getSrcTableName()
                        + " where " + this.dataMapper.getSrcWhere()
                        + " and " + this.dataMapper.getSrcGroupColumns() + " in ( "
                        + " select * from ( "
                        + " select " + this.dataMapper.getSrcGroupColumns()
                        + " from " + this.dataMapper.getSrcTableName() + " model "
                        + " where " + this.dataMapper.getSrcWhere()
                        + " group by " + this.dataMapper.getSrcGroupColumns()
                        + " order by " + this.dataMapper.getSrcOrderBy() + " limit "
                        + this.dataMapper.getSrcGetLimit() + " ) as ds ) order by "+ this.dataMapper.getSrcOrderBy();
            }
        }

        return sqlString;
    }

    /**
     * 执行脚本获取原始数据
     * @return 原始数据
     * @throws Exception 脚本异常
     */
    private List<Map<String, Object>> getDataByScript() throws Exception {
        String scriptNo = this.dataMapper.getScriptNo();
        Object returnObj;

        // 如果脚本编号配置的是 groovy 文件，就直接调用文件，便于运行中打断点调试
        if (scriptNo.indexOf(".groovy") > 0) {
            returnObj = engineScriptManager.invokeGroovyScript(
                    scriptNo,
                    WmsSAPDbConstWords.SCRIPT_PULL_FUN_NAME,
                    this.dataMapper);
        } else {
            returnObj = engineScriptManager.invokeFuncation(
                    this.dataMapper.getOrganizeCode(),
                    scriptNo,
                    WmsSAPDbConstWords.SCRIPT_PULL_FUN_NAME,
                    this.dataMapper);
        }

        if (!(returnObj instanceof List)) {
            throw new RuntimeException("Script pull data result object type error !");
        }

        return (List<Map<String, Object>>)returnObj;
    }

    /**
     * 执行脚本过滤原始数据
     * @param srcData 原始数据
     * @return 过滤后的原始数据
     * @throws Exception 脚本异常
     */
    private List<Map<String, Object>> getDataByFilter(List<Map<String, Object>> srcData) throws Exception {
        String scriptNo = this.dataMapper.getScriptNo();
        Object returnObj;

        // 如果脚本编号配置的是 groovy 文件，就直接调用文件，便于运行中打断点调试
        if (scriptNo.indexOf(".groovy") > 0) {
            returnObj = engineScriptManager.invokeGroovyScript(
                    scriptNo,
                    WmsSAPDbConstWords.SCRIPT_FILTER_FUN_NAME,
                    this.dataMapper, srcData);
        } else {
            returnObj = engineScriptManager.invokeFuncation(
                    this.dataMapper.getOrganizeCode(),
                    this.dataMapper.getScriptNo(),
                    WmsSAPDbConstWords.SCRIPT_FILTER_FUN_NAME,
                    this.dataMapper, srcData);
        }

        if (!(returnObj instanceof List)) {
            throw new RuntimeException("Script filter data result object type error !");
        }

        return (List<Map<String, Object>>)returnObj;
    }

    /**
     * 判断数据库是不是 MSSQL
     * @return
     * @throws Exception
     */
    private boolean isMSSQL() throws Exception {
        return srcConn
                .getMetaData().getURL().contains("sqlserver");
    }

    /**
     * 读取数据（DataTable 格式）
     * @return
     */
    public DataTable readDataTable() {
        DataTable dataTable = null;

        try {
            dataTable = new DataTable(this.dataMapper.getSrcTableName(), this.readData());
        } catch (Exception e) {
            e.printStackTrace();
        }

        return dataTable;
    }

    /**
     * 重写判断对象相等的方法
     * @param obj
     * @return
     */
    @Override
    public boolean equals(Object obj) {
        return super.equals(obj);
    }

    /**
     * 重写 hashCode 方法
     * @return
     */
    @Override
    public int hashCode() {
        return super.hashCode();
    }
}
