package cn.estsh.i3plus.ext.mes.apiservice.dbinterface;

import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper;
import cn.estsh.i3plus.pojo.wms.dbinterface.MappingItem;
import cn.estsh.i3plus.pojo.wms.dbinterface.UpdateSyncItem;
import com.alibaba.fastjson.JSONObject;
import lombok.Data;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * @Description : 接口数据同步基类
 * @Reference :
 * @Author : rock.yu
 * @CreateDate : 2019-06-11 19:34
 * @Modify:
 * 1.
 **/
@Data
@Component
public class WmsSAPDbSyncBase {

    /**
     * 数据映射对象，对应接口库的一张表
     */
    protected MesInterfaceDataMapper dataMapper;
    /**
     * 源表的排序字段
     */
    protected List<String> scrOrderBy;
    /**
     * 源表的查询条件
     */
    protected List<String> srcWhere;
    /**
     * 源表同步成功后需要更新的字段以及值（用表达式替换变量值）
     */
    protected List<UpdateSyncItem> srcUpdateSync;
    /**
     * 源表的主键
     */
    protected List<String> srcPkColumn;
    /**
     * 如果此属性不为空，则把源记录复制多份到每个组织下面
     */
    protected List<String> copyByOrg;
    /**
     * 目标对象的名称（或者表名）
     */
    protected List<String> destBeanName;
    /**
     * 目标表的业务主键，用于判断记录是否存在
     */
    protected List<String> destPkProperty;
    /**
     * 源表与目标表的字段映射关系，当源表中不存在此字段时用配置的默认值写入到目标表
     */
    protected List<MappingItem> mappingItem;

    public WmsSAPDbSyncBase() { }

    public WmsSAPDbSyncBase(MesInterfaceDataMapper dataMapper) {
        setDataMapper(dataMapper);
    }

    public void setDataMapper(MesInterfaceDataMapper dataMapper) {
        this.dataMapper = dataMapper;

        this.setScrOrderBy(getSplitArray(dataMapper.srcOrderBy));
        this.setSrcWhere(getSplitArray(dataMapper.srcWhere));
        this.setSrcUpdateSync(getJSONItems(dataMapper.srcUpdateSync, UpdateSyncItem.class));
        this.setSrcPkColumn(getSplitArray(dataMapper.srcPkColumns));
        this.setCopyByOrg(getSplitArray(dataMapper.copyByOrgs));
        this.setDestPkProperty(getSplitArray(dataMapper.destPkProperties));
        this.setMappingItem(sortedMappingItem(getJSONItems(dataMapper.destColumnMapping, MappingItem.class)));
        if (this.getMappingItem() == null) {
            this.setMappingItem(new ArrayList<>());
        }
        // 依赖于 this.mappingItem 的值，赋值顺序不能变
        this.setDestBeanName(this.getDistinctDestBeanName());
    }


    /**
     * 根据目标表的名称获取主键列表
     * @param beanName 目标表的名称
     * @return 主键列表
     */
    public List<String> getDestPkByBeanName(String beanName) {
        List<String> pkList = new ArrayList<>();

        for (MappingItem item : this.getMappingItem()) {
            if (item.getDestBeanName().equals(beanName) && isPk(item)) {
                pkList.add(item.getDestName());
            }
        }

        return pkList;
    }

    /**
     * 判断该字段是不是主键
     * @param item 自对对照关系
     * @return 是否主键
     */
    private Boolean isPk(MappingItem item) {
        return item.getDestPk() != null && item.getDestPk() == 1;
    }

    /**
     * 从字段映射关系中获取所有的目标表名
     * @return
     */
    private List<String> getDistinctDestBeanName() {
        Map<String, String> beanNamesMap = new HashMap<>();

        for (MappingItem item : this.getMappingItem()) {
            if (!beanNamesMap.containsKey(item.getDestBeanName())) {
                beanNamesMap.put(item.getDestBeanName(), null);
            }
        }

        return new ArrayList<String>(beanNamesMap.keySet());
    }

    /**
     * 把半角逗号分隔的字符串转换成字符串列表
     * @param text 半角逗号分隔的字符串
     * @return 分割后的字符串数组
     */
    private List<String> getSplitArray(String text) {
        if (text == null || text.isEmpty()) {
            return null;
        }

        String[] items = text.split(",");

        return Arrays.asList(items);
    }

    /**
     * 获取字段映射列表
     * @param jsonString JSON 字符串
     * @return 字段映射列表
     */
    public <T> List<T> getJSONItems(String jsonString, Class<T> objClass) {
        if (jsonString == null || jsonString.isEmpty()) {
            return null;
        }

        return JSONObject.parseArray(jsonString, objClass);
    }

    /**
     * 根据 seq 对 MappingItem 列表进行排序
     * @param mappingItems 字段映射关系集合
     * @return 排序后的字段映射关系集合
     */
    public List<MappingItem> sortedMappingItem(List<MappingItem> mappingItems) {
        if (mappingItems == null) {
            return null;
        }

        if (mappingItems.size() <= 1) {
            return mappingItems;
        }

        Collections.sort(mappingItems);

        return mappingItems;
    }

    /**
     * 安全获取整数值
     * @param obj
     * @return
     */
    public Integer getSafetyInteger(Object obj) {
        if (obj == null) {
            return 0;
        }

        return Integer.valueOf(String.valueOf(obj));
    }

    /**
     * 安全获取整数值
     *
     * @param obj
     * @return
     */
    public Long getSafetyLong(Object obj) {
        if (obj == null) {
            return 0L;
        }

        return Long.valueOf(String.valueOf(obj));
    }
    /**
     * 安全获取字符串值
     * @param obj
     * @return
     */
    public String getSafetyString(Object obj) {
        if (obj == null) {
            return "";
        }

        return String.valueOf(obj);
    }

    /**
     * 重写判断对象相等的方法
     * 用 JSON 字符串比较可能会性能比较低，逐个字段判断又太繁琐
     * @param obj
     * @return
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }

        if (obj instanceof WmsSAPDbSyncBase) {
            WmsSAPDbSyncBase newObj = (WmsSAPDbSyncBase)obj;

            if (JSONObject.toJSONString(this).
                    equals(JSONObject.toJSONString(newObj))) {
                return true;
            }
        }

        return false;
    }

    /**
     * 重写 hashCode 方法，实际上这个对象不会作为 key 放到 Hash 中
     * 用 JSON 字符串比较可能会性能比较低，逐个字段判断又太繁琐
     * @return
     */
    @Override
    public int hashCode() {
        return JSONObject.toJSONString(this).hashCode();
    }
}
