package cn.estsh.i3plus.ext.mes.apiservice.dbinterface;

import cn.estsh.i3plus.mes.apiservice.util.datatable.DataRow;
import cn.estsh.i3plus.mes.apiservice.util.datatable.DataSet;
import cn.estsh.i3plus.mes.apiservice.util.datatable.DataTable;
import cn.estsh.i3plus.mes.apiservice.util.datatable.DataTypes;
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper;
import cn.estsh.i3plus.pojo.wms.dbinterface.MappingItem;
import com.alibaba.fastjson.JSONObject;
import lombok.Getter;
import lombok.Setter;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description : 接口数据转换
 * 按照配置规则把读取的接口数据转换成业务数据
 * @Reference :
 * @Author : rock.yu
 * @CreateDate : 2019-06-11 19:34
 * @Modify:
 * 1.
 **/
@Component
public class WmsSAPDbTranslator extends WmsSAPDbSyncBase {


    @Getter
    @Setter
    private WmsSAPDbExpression wmsSAPDbExpression;

    public WmsSAPDbTranslator() { }

    public WmsSAPDbTranslator(WmsSAPDbExpression wmsSAPDbExpression) {
        this.wmsSAPDbExpression = wmsSAPDbExpression;
    }

    public WmsSAPDbTranslator(MesInterfaceDataMapper dataMapper) {
        super(dataMapper);
    }

    /**
     * 把原始数据表按照字段对应关系转换成目标数据集（存在一对多的关系）
     * @param originData
     * @return
     * @throws Exception
     */
    public DataSet translateData(List<Map<String, Object>> originData) throws Exception {
        if (this.dataMapper == null) {
            throw new IllegalArgumentException("not set data mapper!");
        }

        // 原始数据表
        DataTable originTable = new DataTable(this.dataMapper.srcTableName, originData);

        // 构建目标数据集
        DataSet destDataSet = buildDestDataSet(this.getDestBeanName(), this.getMappingItem());

        for(DataRow row : originTable.getRows()) {
            // 拆分数据行
            Map<String, DataRow> destRowsMap = splitDataRow(destDataSet, row);

            // 把拆分好的数据行放到目标数据表中
            for(DataTable table : destDataSet.getDataTables()) {
                if (destRowsMap.containsKey(table.getTableName())) {
                    table.addRow(destRowsMap.get(table.getTableName()));
                }
            }
        }

        return destDataSet;
    }

    /**
     * 把原始数据行拆分成目标数据行，存在一对多的情况
     * @param dataSet
     * @param originRow
     * @return
     * @throws Exception
     */
    private Map<String, DataRow> splitDataRow(DataSet dataSet, DataRow originRow) throws Exception {
        // 最终返回的 Map
        Map<String, DataRow> rowsMap = new HashMap<>();

        // 构建临时数据行对象
        for (DataTable table : dataSet.getDataTables()) {
            rowsMap.put(table.getTableName(), table.newRow());
        }

        // 遍历字段映射列表
        for (MappingItem item : this.getMappingItem()) {
            // 原始表中有这个字段
            if (originRow.getColumns().contains(item.getSrcName())) {
                // 把来源数据行的值放到对应的目标数据行中
                rowsMap.get(item.getDestBeanName()).
                        setValue(item.getDestName(), originRow.getValue(item.getSrcName()));
            } else {
                // 原始表中没有这个字段，用预设的默认值填充此字段
                rowsMap.get(item.getDestBeanName()).
                        setValue(item.getDestName(), item.getDefaultValue());
            }
        }

        // 遍历第二遍，进行默认值的表达式计算，此时所有属性都已赋值，在配置字段映射关系就可以不用管属性的配置顺序
        // 如果强制要求配置字段映射关系时把有表达式的属性放在后面，也可以不用再循环一遍，直接在上面的循环解析即可
        for (MappingItem item : this.getMappingItem()) {
            if (item.getDefaultValue() != null // 不是空对象
                    && (item.getDefaultValue() instanceof String) // 字符串类型
                    && !item.getDefaultValue().toString().isEmpty() // 字符串不为空
                    && item.getDefaultValue().toString().indexOf("#") > -1) { // 包含表达式


                // 取出当前字段的值
                //Object valueObj = rowsMap.get(item.getDestBeanName()).getValue(item.getDestName());
                // 把当前行的数据（原始数据行和目标数据行都参与计算）和表达式进行计算
                Object valueObj = parseDefaultValue(
                        combineRowMap(originRow.getItemMap(), rowsMap.get(item.getDestBeanName()).getItemMap()),
                        item.getDefaultValue());
                // 把计算后的值设置回数据行中
                rowsMap.get(item.getDestBeanName()).setValue(item.getDestName(), valueObj);
            }
        }

        return rowsMap;
    }

    /**
     * 合并来源表的所有字段和目标表的所有字段
     * 为了防止字段重复，来源表的字段名统一加上 _o 后缀(暂时不加后缀，因为两套系统的字段编码格式很少相同)
     * @param originMap 来源表的字段
     * @param destMap 目标表的字段
     * @return 合并后的 Map
     */
    private Map<String, Object> combineRowMap(Map<String, Object> originMap, Map<String, Object> destMap) {
        Map<String, Object> combineMap = new HashMap<>();

//        if (originMap != null && originMap.size() > 0) {
//            for(Map.Entry<String, Object> entry : originMap.entrySet()) {
//                // 来源表的字段名加上 _o 后缀，防止跟目标表的字段重名
//                combineMap.put(entry.getKey() + "_o", entry.getValue());
//            }
//        }

        if (originMap != null && originMap.size() > 0) {
            combineMap.putAll(originMap);
        }

        if (destMap != null && destMap.size() > 0) {
            combineMap.putAll(destMap);
        }

        return combineMap;
    }

    /**
     * 构建目标数据集（可能存在一张原始表对应多张目标表）
     * @param destBeanNames
     * @param mappingItems
     * @return
     * @throws Exception
     */
    private DataSet buildDestDataSet(List<String> destBeanNames, List<MappingItem> mappingItems) throws Exception {
        // 目标数据集
        DataSet destDataSet = new DataSet();

        // 构建目标表
        for(String tableName : destBeanNames) {
            DataTable table = new DataTable(tableName);

            // 构建目标表的结构
            for(MappingItem item : mappingItems) {
                // 当前字段属于这张表，并且目标字段不为空
                if (item.getDestBeanName().equals(tableName) && item.destName != null && !item.destName.isEmpty()) {
                    // 加入字段
                    // 此处暂不进行表达式计算
                    table.addColumn(item.getDestName(), DataTypes.getDataType(item.getDefaultValue()));
                }
            }

            // 把数据表加入数据集
            destDataSet.add(table);
        }

        return destDataSet;
    }

    /**
     * 根据表达式计算默认值
     * @param rowMap 当前行的数据 Map
     * @param defaultValue 原始默认值（可能包含表达式）
     * @return 计算后的默认值
     */
    private Object parseDefaultValue(Map<String, Object> rowMap, Object defaultValue) throws Exception {
        if (defaultValue == null) {
            return null;
        }

        // 不是字符串类型，直接返回
        if (!(defaultValue instanceof String)) {
            return defaultValue;
        }

        // 没有包含表达式，直接返回
        if (defaultValue.toString().indexOf("#") == -1) {
            return defaultValue;
        }

        Object objResult;

        try {
            objResult = wmsSAPDbExpression.parse(defaultValue.toString(), rowMap);
        } catch (Exception e) {
            e.printStackTrace();

            throw new IllegalArgumentException("表达式计算异常，请检查表达式语法是否正确！ exp: "
                    + defaultValue.toString() + " Exception: "
                    + e.getMessage() + " rowMap:"
                    + JSONObject.toJSONString(rowMap));
        }

        // 计算后返回
        return objResult;
    }

    /**
     * 重写判断对象相等的方法
     * @param obj
     * @return
     */
    @Override
    public boolean equals(Object obj) {
        return super.equals(obj);
    }

    /**
     * 重写 hashCode 方法
     * @return
     */
    @Override
    public int hashCode() {
        return super.hashCode();
    }
}
