package cn.estsh.i3plus.ext.mes.apiservice.dbinterface;

import cn.estsh.i3plus.icloud.softswitch.sdk.IBsSuitServiceCloud;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.enumutil.BlockSoftSwitchEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.softswitch.bean.BsSuitCase;
import cn.estsh.i3plus.pojo.softswitch.bean.BsSuitCaseParam;
import cn.estsh.i3plus.pojo.softswitch.bean.BsSuitDataDetail;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import lombok.NoArgsConstructor;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
@NoArgsConstructor
public class WmsSAPDbUtil {
    private static final Logger LOGGER = LoggerFactory.getLogger(WmsSAPDbUtil.class);

    // 软适配微服调用代理对象
    @Autowired
    IBsSuitServiceCloud bsSuitServiceCloud;


    /**
     * 从 XML 格式的报文中抽取数据
     * @param xmlString XML 报文
     * @param tableList XML 报文中的数据对象结构，要按从大到小排序
     * @return 抽取到的数据，扁平化的 List<Map<String, Object>> 格式
     * @throws Exception XML 解析异常
     */
    public List<Map<String, Object>> convertXmlToList(String xmlString, List<String> tableList)
            throws Exception {
        Document document = DocumentHelper.parseText(xmlString);
        Element rootElement = document.getRootElement();
        List<Element> elements = rootElement.selectNodes("//*");

        List<Map<String, Object>> rows = new ArrayList<>();

        for (Element element : elements) {
            if (tableList.get(0).equals(element.getName())) {
                Map<String, Object> row = new HashMap<>();
                parseElement(element, row, rows, tableList);
            }
        }

        return rows;
    }

    /**
     * 从 XML 格式的报文中抽取数据
     * @param xmlString XML 报文
     * @param tables 数据结构的层级，用半角逗号分隔
     * @return 抽取到的数据，扁平化的 List<Map<String, Object>> 格式
     * @throws Exception XML 解析异常
     */
    public List<Map<String, Object>> convertXmlToList(String xmlString, String tables)
            throws Exception {
        List<String> tableList = this.getListByString(tables);

        if (tableList == null || tableList.size() == 0) {
            return null;
        }

        return convertXmlToList(xmlString, tableList);
    }

    /**
     * 把 List<Map<String, Object> 格式的数据转换成 XML 字符串
     * @param listData 原始数据
     * @param rootNodeName 根节点名称
     * @param nodeName 叶子节点名称
     * @param rootClassName 根节点的类名
     * @param nodeClassName 叶子节点的类名
     * @return XML 字符串
     */
    public String convertListToXml(List<Map<String, Object>> listData, String rootNodeName, String nodeName,
                                   String rootClassName, String nodeClassName) {
        if (listData == null || listData.size() == 0) {
            throw new RuntimeException("listData is null ! ");
        }

        StringBuilder sb = new StringBuilder();

        if (rootNodeName != null && !rootNodeName.isEmpty()) {
            sb.append(String.format("<%s class=\"%s\">\n", rootNodeName, rootClassName));
        }

        for(Map<String, Object> row : listData) {
            sb.append(String.format("<%s class=\"%s\">\n", nodeName, nodeClassName));

            for (Map.Entry<String, Object> entry : row.entrySet()) {
                sb.append(String.format("<%s>%s</%s>\n", entry.getKey(), entry.getValue(), entry.getKey()));
            }

            sb.append(String.format("</%s>\n", nodeName));
        }

        if (rootNodeName != null && !rootNodeName.isEmpty()) {
            sb.append(String.format("</%s>\n", rootNodeName));
        }

        return sb.toString();
    }

    /**
     * 把报文的 ID 加入到数据集中（增加一列 SID）
     * @param listData 原始数据集
     * @param sid 报文 ID
     */
    public void packSID(List<Map<String, Object>> listData, Long sid) {
        if (listData == null || listData.size() == 0) {
            return;
        }

        for (Map<String, Object> row : listData) {
            row.put("SID", sid);
        }
    }

    /**
     * 把半角逗号分隔的字符串转成 List<String> 格式
     * @param strList 半角逗号分隔的字符串
     * @return List<String>
     */
    public List<String> getListByString(String strList) {
        if (strList == null || strList.isEmpty()) {
            return null;
        }

        String[] arrList = strList.split(",");

        return Arrays.asList(arrList);
    }

    /**
     * 递归函数，用户解析 XML 报文并抽取数据
     * @param element XML 元素
     * @param row 当前数据行
     * @param rows 数据行列表
     * @param tableList 数据对象的结构
     */
    private void parseElement(Element element, Map<String, Object> row,
                              List<Map<String, Object>> rows, List<String> tableList) {

        List<Element> childElements = element.elements();

        if (childElements != null) {
            // 先遍历没有子节点的属性，因为有属性值在子节点之后，所以必须先遍历一遍
            for (Element eleData : childElements) {
                if (eleData.elements() == null || eleData.elements().size() == 0) {
                    row.put(element.getName() + "_" + eleData.getName(), eleData.getStringValue().trim());
                }
            }

            // 再遍历包含子节点的属性
            for (Element ele : childElements) {
                if (tableList.contains(ele.getName()) || ele.getName().startsWith("List")) {
                    parseElement(ele, row, rows, tableList);
                }
            }

            // 末尾元素，开始新增行
            if (element.getName().equals(tableList.get(tableList.size() - 1))) {
                HashMap<String, Object> newRow = new HashMap<>();
                newRow.putAll(row);
                rows.add(newRow);
            }
        }
    }

    /**
     * 从软适配中按照适配编号获取报文（微服务调用）
     * @param suitCaseCode 适配编号
     * @param packageCount 获取报文的数量
     * @return 报文对象
     */
    public ResultBean readSoftSwitchPackage(String suitCaseCode, Integer packageCount) {
        BaseResultBean<BsSuitDataDetail> resultBean =
                bsSuitServiceCloud.findLastUnProcessBsSuitDataDetailBySuitCaseCode(suitCaseCode, packageCount);

        if (resultBean.isSuccess()) {
            List<BsSuitDataDetail> bsSuitDataDetailList = resultBean.getResultList();

            return new ResultBean(true).setResultObject(bsSuitDataDetailList);
        } else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.WMS.getCode())
                    .setErrorCode(ImppExceptionEnum.CLOUD_CALL_EXCEPTION.getCode())
                    .setErrorDetail("微服调用失败: " + resultBean.getErrorMsg())
                    .build();
        }
    }

    /**
     * 按照报文编号调用软适配，把报文标记为已处理
     * @param packageIds 报文编号
     * @return 标记结果
     */
    public boolean ackSoftSwitchPackage(Long[] packageIds) {
        BaseResultBean<BsSuitDataDetail> resultBean =
                bsSuitServiceCloud.updateBsSuitDataDetailProcessStateByIds(
                        packageIds, BlockSoftSwitchEnumUtil.SUIT_PROCESS_STATUS.PROCESS.getValue());

        if (resultBean.isSuccess()) {
            return true;
        } else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.WMS.getCode())
                    .setErrorCode(ImppExceptionEnum.CLOUD_CALL_EXCEPTION.getCode())
                    .setErrorDetail("微服调用失败: " + resultBean.getErrorMsg())
                    .build();
        }
    }

    /**
     * 调用微服务接口发送报文
     * @param suitCaseCode 适配编号
     * @param xmlString XML 报文
     */
    public void sendSoftSwitchPackage(String suitCaseCode, String xmlString) {
        LOGGER.info("WmsSAPDbUtil.sendSoftSwitchPackage input:suitCaseCode：{} ;xmlString:{}",suitCaseCode,xmlString);
        BsSuitCase bsSuitCase = new BsSuitCase();
        // 设置适配器代码
        bsSuitCase.setSuitCaseCode(suitCaseCode);
        bsSuitCase.setBsSuitCaseParamList(new ArrayList<>());
        bsSuitCase.setParamXml(xmlString);

        BaseResultBean<BsSuitDataDetail> resultBean =
                bsSuitServiceCloud.executeBsSuitCase(bsSuitCase);
        LOGGER.info("bsSuitServiceCloud.executeBsSuitCase result:{}", resultBean);
        if (!resultBean.isSuccess()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.WMS.getCode())
                    .setErrorCode(ImppExceptionEnum.CLOUD_CALL_EXCEPTION.getCode())
                    .setErrorDetail("bsSuitServiceCloud.executeBsSuitCase failed: %s; suitCaseCode:%s; xmlString:%s", resultBean.getErrorMsg(), suitCaseCode, xmlString)
                    .build();
        }
    }

    /**
     * 以带参数名的方式调用微服务接口发送报文
     *
     * @param suitCaseCode 适配编号
     * @param xmlString    XML 报文
     */
    public void sendSoftSwitchPackageWithParams(String suitCaseCode, String xmlString) {
        BsSuitCase bsSuitCase = new BsSuitCase();
        // 设置适配器代码
        bsSuitCase.setSuitCaseCode(suitCaseCode);

        List<BsSuitCaseParam> bsSuitCaseParamList = new ArrayList<>();
        BsSuitCaseParam bsSuitCaseParam = new BsSuitCaseParam();
        bsSuitCaseParam.setParamName("params");
        bsSuitCaseParam.setParamValue(xmlString);
        bsSuitCaseParamList.add(bsSuitCaseParam);
        bsSuitCase.setBsSuitCaseParamList(bsSuitCaseParamList); // 多个数据

        BaseResultBean<BsSuitDataDetail> resultBean =
                bsSuitServiceCloud.executeBsSuitCase(bsSuitCase);
        LOGGER.info("bsSuitServiceCloud.executeBsSuitCase result:{}",resultBean);
        if (!resultBean.isSuccess()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.WMS.getCode())
                    .setErrorCode(ImppExceptionEnum.CLOUD_CALL_EXCEPTION.getCode())
                    .setErrorDetail("bsSuitServiceCloud.executeBsSuitCase failed: %s; suitCaseCode:%s; xmlString:%s",resultBean.getErrorMsg(), suitCaseCode, xmlString)
                    .build();
        }
    }

    /**
     * 以带参数名的方式调用微服务接口发送报文
     *
     * @param suitCaseCode 适配编号
     * @param resultMap     发送的数据
     */
    public void sendSoftSwitchPackageWithMap(String suitCaseCode, Map<String, Object> resultMap) {
        BsSuitCase bsSuitCase = new BsSuitCase();
        // 设置适配器代码
        bsSuitCase.setSuitCaseCode(suitCaseCode);

        List<BsSuitCaseParam> bsSuitCaseParamList = new ArrayList<>();
        Iterator<String> iterator = resultMap.keySet().iterator();
        while(iterator.hasNext()) {
            String key = iterator.next();
            BsSuitCaseParam bsSuitCaseParam = new BsSuitCaseParam();
            bsSuitCaseParam.setParamName(key);
            bsSuitCaseParam.setParamValue(resultMap.get(key));
            bsSuitCaseParamList.add(bsSuitCaseParam);
        }
        bsSuitCase.setBsSuitCaseParamList(bsSuitCaseParamList); // 多个数据
        BaseResultBean<BsSuitDataDetail> resultBean =
                bsSuitServiceCloud.executeBsSuitCase(bsSuitCase);
        LOGGER.info("bsSuitServiceCloud.executeBsSuitCase result:{}",resultBean);
        if (!resultBean.isSuccess()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.WMS.getCode())
                    .setErrorCode(ImppExceptionEnum.CLOUD_CALL_EXCEPTION.getCode())
                    .setErrorDetail("bsSuitServiceCloud.executeBsSuitCase failed: %s; suitCaseCode:%s, data:%s", resultBean.getErrorMsg(), suitCaseCode, resultMap.toString())
                    .build();
        }
    }

}
